

#include <SoftwareSerial.h>

#define LCD_pin 2

/*

pin reference:
==============

digital:
  0 rx 
  1 tx - MIDI
  2 lcd
  3 key 0 - bank switch / edit
  4 key 1 - OK
  5 key 2
  6 key 3
  7 key 4
  8 key 5
  9 key 6
 10 key 7
 11 key 8
 12 key 9
 13 LED connected internally - too much trouble for using it as an input
 
 
 
analog:
  1 knob 1
  2 knob 2
  3 knob 3
  4 knob 4
  5 knob 5
  6 knob 6 / edit value change
  
commands sent to PC: 
   0-9 keys
  10-15 knobs
*/

SoftwareSerial swSerial(LCD_pin, LCD_pin);


short ctls_d[10]={1,1,1,1,1,1,1,1,1, 1};
short ctls_d_old[10]={0,0,0,0,0,0,0,0,0,0};

short ctls_a[6]={1,1,1,1,1,1};
short ctls_a_old[6]={0,0,0,0,0,0};

short led_blink=0;
short led_intensity=3;

short bank_switch=1;
short bank=1;

void setup(){
  pinMode(LCD_pin, OUTPUT);

  pinMode(0, OUTPUT);
  pinMode(1, INPUT);
  pinMode(13, OUTPUT);

  Serial.begin(115200);  
//  Serial.begin(31250);  
  swSerial.begin(9600);
  
  if(digitalRead(3)==LOW){
    swSerial.print(" (EDIT mode)");

    // add EDIT mode

  }
  
  // setup pins
  int i;
  for(i=3;i<13;i++){
    pinMode(i, INPUT);
    digitalWrite(i, HIGH);
  }

  for(i=0;i<6;i++){
    ctls_a[i]=analogRead(i)>>3;
    ctls_a_old[i]=ctls_a[i];
  }

  Serial.print('s', BYTE);  // ready to go  

  // wait for a little bit (just a precaution) and
  // then send current status of keys and knobs  
  delay(100);
  transmit_status();
}


void read_data(){
  int i;
  int val;

  // read digital pins
  for(i=3;i<13;i++){
    val=digitalRead(i);
    if(val!=ctls_d[i-3]){
      ctls_d[i-3]=val;
      if(val==0){
        val=127;
        led_blink=1;
        if(i==3) bank_switch=1;  // mark bank switch
      }
      else
        val=0;
      Serial.print(i-3, BYTE);
      Serial.print(val, BYTE);
    }
  }
  
  // read analog pins
  for(i=0;i<6;i++){
    val=((analogRead(i)>>3)+ctls_a_old[i])/2;  // simple lowpass filtering to remove value oscillations
    if(val==126) val=127;
    
    if(val!=ctls_a[i]){
      ctls_a[i]=val;
      Serial.print(i+10, BYTE);
      Serial.print(val, BYTE);
      led_blink=1;
    }
    ctls_a_old[i]=val;
  }
}


void transmit_status(){
  int i;
  for(i=0;i<10;i++){
    Serial.print(i, BYTE);
    Serial.print(ctls_d[i], BYTE);
  }
  for(i=0;i<6;i++){
    Serial.print(i+10, BYTE);
    Serial.print(ctls_a[i], BYTE);
  }

  delay(20);
  swSerial.print("?f");
  delay(20);
  swSerial.print("Ctrlr v0.1");
  delay(20);
}  

void loop(){
  int val, i;
  
  read_data();
  
  // blink LED on new event  
  if(led_blink==1) digitalWrite(13, HIGH);
  if(led_blink>0) led_blink++;
  if(led_blink==led_intensity){
    digitalWrite(13, LOW);
    led_blink=0;
  }
  
  if(bank_switch){
    // mark current bank
    bank_switch=0;
    
    if(bank==1) bank=0;
    else bank=1;

    // display current bank on LCD
    swSerial.print("?x15");
    swSerial.print("?y0");
    (bank==1)?swSerial.print(" "):swSerial.print("*");
    swSerial.print("?x15");
    swSerial.print("?y1");
    (bank==1)?swSerial.print("*"):swSerial.print(" ");    
  }

}
